--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--             PAQUETAGE DES ARBRES BALANCES                --
--                                                          --
-- Note: les arbres generaux de recherche sont des arbres   --
--    <o,f> ou o noeud = liste_ordonnee de valeurs,         --
--             f foret = liste d'arbres generaux            --
--    tels que:                                             --
--        si estvide (f) alors est-triee (o)                --
--        sinon longueur (f) = longueur (o) + 1 et          --
--            si o = [e] et f = [a;b] alors                 --
--                max(a) <= la_cle(o) <= min (b)            --
--            sinon soit o = e :: o1, f = a :: f1,          --
--                max(a) <= la_cle(e) <= min(ieme(f1,1))    --
--                et <o1,f1> est un arbre general recherche --
--            fsi                                           --
--        fsi                                               --
--    avec:                                                 --
--    max(<o,f>) =                                          --
--        si f=vide alors la_cle(ieme(o, longueur(o))       --
--        sinon max(ieme(f, longueur(f)) fsi                --
--    min(<o,f>) =                                          --
--        si f=vide alors la_cle(ieme(o, 1))                --
--        sinon min(ieme(f, 1)) fsi                         --
--    Le paquetage implante la representation des noeuds    --
--        dun arbre balance sous forme de deux listes:     --
--        - Contenu est la liste ordonnees des valeurs      --
--        - SousArbres est la liste des identifiants        --
--          darbres                                        --
--        Les operations permettent la manipulation des     --
--        listes et les transformations des noeuds          --
-- preconditions                                            --
--    En plus des preconditions sur les listes, on aura     --
--    Inserer (Le_Noeud,Indice,Val): Est_Feuille(Le_Noeud)  --
--    Inserer (Le_Noeud,Indice,Val,Q):                      --
--        non Est_Feuille(Le_Noeud)                         --
--    Transfert_Gauche_Droite(n,I,G,D),                     --
--    Transfert_Droite_Gauche(n,I,G,D), Regrouper(n,I,G,D): --
--        Est_Feuille(G) = Est_Feuille(D)                   --
-- proprietes fonctionnelles                                --
--    Initialiser_Feuille (Le_Noeud, Val) initialise le     --
--        Contenu avec Val, et Sous_Arbres a vide           --
--    Initialiser_Noeud (Le_Noeud,Val_Separ,Gauche,Droite)  --
--        initialise le Contenu a Val_Separ, et Sous_Arbres --
--        avec Gauche et Droite                             --
--    Longueurs_Compatibles (Le_Noeud) =                    --
--        longueur(Sous_Arbres) = 0 ou longueur(Contenu)+1  --
--    Degre (Le_Noeud) = longueur (Contenu)                 --
--    Est_Feuille (Le_Noeud) = longueur (Sous_Arbres) = 0   --
--    Rang (Le_Noeud, La_Cle) retourne la position occupee  --
--        ou qui devrait etre occupee par l'element de cle  --
--        La_Cle dans Contenu                               --
--    Ieme_Valeur(Le_Noeud, Indice) = Ieme(Contenu, Indice) --
--    Ieme_Fils (Le_Noeud,Indice) = Ieme(Sous_Arbres,Indice)--
--    Inserer (Le_Noeud, Indice, Val) execute               --
--        Inserer(Contenu, Indice, Val)                     --
--    Inserer (Le_Noeud, Indice, Val, Q) execute            --
--        Inserer(Contenu, Indice, Val) et                  --
--        Inserer(Sous_Arbres, Indice+1, Q)                 --
--    Supprimer (Le_Noeud, Indice) execute                  --
--        Supprimer(Contenu, Indice) et eventuellement      --
--        Supprimer(Sous_Arbres, Indice+1)                  --
--    Changer_Valeur (Le_Noeud, Indice, Val) execute
--        Changer_Ieme(Contenu, Indice, Val)                --
--    Transfert_Gauche_Droite(Le_Noeud_Pere,Indice,Gauche,  --
--        Droite) transfere une valeur depuis le noeud de   --
--        Gauche vers celui de Droite par le biais de celle --
--        en Indice de Le_Noeud_Pere                        --
--    Transfert_Droite_Gauche(Le_Noeud_Pere,Indice,Gauche,  --
--        Droite) transfere une valeur depuis le noeud de   --
--        Droite vers celui de Gauche par le biais de celle --
--        en Indice de Le_Noeud_Pere                        --
--    Regrouper (Le_Noeud_Pere, Indice, Gauche, Droite)     --
--        rassemble les valeurs de Droite dans Gauche, avec --
--        la valeur en Indice de Le_Noeud_Pere              --
--    Eclater (Le_Noeud, En, Val_Separ, Nouveau) transfere  --
--        les valeurs et sous arbres, provenant de Le_Noeud --
--        et situees apres En, vers Nouveau, et met dans    --
--        Val_Separ la valeur qui etait situee en En        --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Listes_Contigues_Ordonnees;
with Listes_Contigues;
generic
  Degre_Max : in Positive := 100;
  type Id_Noeud is private;
  type Element is private;
  type Cle is private;
  with function La_Cle (E : in Element) return Cle is <>;
  with function "<"(U, V: in Cle) return Boolean is <>;
package Arbres_Balances is
  type Type_Noeud is private;
  procedure Initialiser_Feuille(Le_Noeud: in out Type_Noeud;
                                 Val : in Element);
  procedure Initialiser_Noeud (Le_Noeud : in out Type_Noeud;
                                Val_Separ : in Element;
                                Gauche, Droite: in Id_Noeud);
  function Longueurs_Compatibles (Le_Noeud : in Type_Noeud) 
                                             return Boolean;
  function Degre (Le_Noeud : in Type_Noeud) return Natural;
  function Est_Feuille (Le_Noeud : in Type_Noeud) 
                                             return Boolean;
  function Rang (Le_Noeud : in Type_Noeud;
                  La_Cle : in Cle) return Positive;
  function Ieme_Valeur(Le_Noeud : in Type_Noeud;
                       Indice : in Positive) return Element;
  function Ieme_Fils (Le_Noeud : in Type_Noeud;
                      Indice : in Positive) return Id_Noeud;
  procedure Inserer (Le_Noeud : in out Type_Noeud;
                      Indice : in Positive;
                      Val : in Element); -- dans une feuille
  procedure Inserer (Le_Noeud : in out Type_Noeud;
                      Indice : in Positive;
                      Val : in Element;
                      Q : in Id_Noeud);         -- noeud interne
  procedure Supprimer (Le_Noeud : in out Type_Noeud;
                        Indice : in Positive);
  procedure Changer_Valeur (Le_Noeud : in out Type_Noeud;
                             Indice : in Positive;
                             Val : in Element);
  procedure Transfert_Gauche_Droite
                        (Le_Noeud_Pere : in out Type_Noeud;
                         Indice : in Positive;
                         Gauche, Droite : in out Type_Noeud);
  procedure Transfert_Droite_Gauche
                        (Le_Noeud_Pere : in out Type_Noeud;
                         Indice : in Positive;
                         Gauche, Droite : in out Type_Noeud);
  procedure Regrouper (Le_Noeud_Pere : in out Type_Noeud;
                        Indice : in Positive;
                        Gauche, Droite : in out Type_Noeud);
  procedure Eclater (Le_Noeud : in out Type_Noeud;
                      En : in Positive;
                      Val_Separ : out Element;
                      Nouveau : in out Type_Noeud);
private
  package L_E is new Listes_Contigues_Ordonnees (Element => Element,
                                                   Cle => Cle);
  subtype Liste_Valeurs is L_E.Liste (Degre_Max);
  package L_P is new Listes_Contigues (Element => Id_Noeud);
  subtype Liste_Fils is L_P.Liste (Degre_Max + 1);
  use L_E, L_P;
  type Type_Noeud is 
    record 
      Contenu : Liste_Valeurs;
      Sous_Arbres : Liste_Fils;
    end record;
end Arbres_Balances;
