--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--             PAQUETAGE DES ARBRES BINAIRES                --
--                         POINTES                          --
--                                                          --
-- Note:1 le paquetage definit la reprsentation des arbres --
--        et un premier ensemble doprations dexploration --
--      2 les noeuds de larbre ne contiennent que les deux --
--        pointeurs vers les fils. Le contenu sera associ  --
--        par extension                                     --
--      3 les positions reperent un noeud et la direction   --
--        prise par rapport a ce noeud                      --
--      4 un arbre est reprsent par un chemin depuis la   --
--        racine jusquau noeud courant, les noeuds etant   --
--        chaines entre eux (gauche et droit)               --
--      5 les operations descendre, raccrocher, etc sont    --
--        necessaires aux paquetages enfants et facilitent  --
--        la gestion des noeuds et des positions            --
--      6 le type Arbre comme le type Noeud sont declares   --
--        abstraits, car cela na pas de sens de creer des  --
--        objets de ces types. Les objest seront dun type  --
--        derive de ceux-la.                                --
-- preconditions                                            --
--    Passer_Au_Suivant[_Prefixe, _Postfixe](L_Arbre),      --
--    Contenu_Courant(L_Arbre), Changer_Courant(L_Arbre)    --
--    Hauteur_Courante(L_Arbre): not A_La_Fin(L_Arbre)      --
--    Descendre(Dans, A_Gauche): longueur du chemin actuel  --
--        infrieure a la hauteur maximum de larbre        --
-- proprietes fonctionnelles                                --
--    Positionner_Sur_Premier[_Prefixe, _Postfixe](L_Arbre) --
--        etablit le chemin vers le premier noeud de la     --
--        liste infixe [resp. prefixe, postfixe]            --
--    Passer_Au_Suivant[_Prefixe, _Postfixe](L_Arbre)       --
--        etablit chemin vers le noeud suivant de la liste  --
--        infixe [resp. prefixe, postfixe] vis a vis du     --
--        noeud courant                                     --
--    Hauteur_Courante(L_Arbre) = longueur du chemin actuel --
--    A_La_Fin (L_Arbre) vrai si le chemin actuel est vide  --
--    Vider (L_Arbre) enleve tous les noeuds de larbre     --
-- operations de service                                    --
--    Descendre(Repere, P, A_Gauche) construit la position  --
--        partant de P, A_Gauche, et change P vers le noeud --
--    Raccrocher (Repere, P) met P comme fils dans la       --
--        direction indiquee par le repere                  --
--    Descendre(Dans, A_Gauche) prolonge le chemin actuel   --
--        dans la direction demandee                        --
--    Raccrocher(Dans, P, En) met P comme noeud, en En sur  --
--        le chemin actuel (par defaut, En est lextremite) --
--    Extremite_Bord_Gauche(L_Arbre) prolonge le chemin     --
--        actuel jusqua lextremite du bord gauche         --
--    Remonter_Bord_Droit(L_Arbre) remonte le chemin actuel --
--        sur la partie correspondant a un bord droit       --
--    Premiere_Feuille(L_Arbre) prolonge le chemin actuel   --
--        jusqua la feuille la plus a gauche               --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 27 mai 1997                       --
--------------------------------------------------------------

with Ada.Unchecked_deallocation;
with Ada.Finalization; 
package Arbres_Binaires is
  type Arbre (Hauteur_Max : Positive) is 
    abstract tagged limited private; 
                 -- procdures dexploration (infixe dabord)
  procedure Positionner_Sur_Premier (L_Arbre : in out Arbre);
  procedure Passer_Au_Suivant (L_Arbre : in out Arbre);
  procedure Positionner_Sur_Premier_Prefixe (L_Arbre: in out Arbre);
  procedure Passer_Au_Suivant_Prefixe(L_Arbre : in out Arbre);
  procedure Positionner_Sur_Premier_Postfixe(L_Arbre: in out Arbre);
  procedure Passer_Au_Suivant_Postfixe(L_Arbre: in out Arbre);
  function A_La_Fin (L_Arbre: in Arbre) return Boolean;
  function Hauteur_Courante (L_Arbre : in Arbre) return Natural;
  pragma Inline (A_La_Fin, Hauteur_Courante); 
  procedure Vider (L_Arbre : in out Arbre);
private 
  type Noeud;
  type Pt_Noeud is access Noeud'Class;
  type Noeud is abstract tagged 
    record
      Gauche, Droite : Pt_Noeud;
    end record;
  type Position is
    record
      Le_Noeud : Pt_Noeud := null;
      A_Gauche : Boolean; -- quelle direction par rapport au noeud
    end record; 
  type Tab_Pt_Noeud is array (Positive range <>) of Position;
  type Arbre (Hauteur_Max : Positive) is abstract
    new Ada.Finalization.Limited_Controlled with
    record
      Chemin : Tab_Pt_Noeud (1..Hauteur_Max);
      Sommet : Natural := 0;
    end record; 
                       -- ne devraient plus tre visibles
                        -- procdures de gestion des positions
  procedure Descendre (Repere : in out Position; 
                        P : in out Pt_Noeud; 
                        A_Gauche : in Boolean);
  procedure Raccrocher (Repere : in Position; P : in Pt_Noeud);
  procedure Descendre (Dans : in out Arbre; 
                        A_Gauche : in Boolean);
  procedure Raccrocher (Dans : in out Arbre; P : in Pt_Noeud;
                         En : in Natural := 0);
  pragma Inline (Descendre, Raccrocher);
                        -- recherche dun nud particulier
  procedure Extremite_Bord_Gauche (L_Arbre: in out Arbre);
  procedure Remonter_Bord_Droit (L_Arbre: in out Arbre);
  procedure Premiere_Feuille (L_Arbre : in out Arbre); 
  procedure Finalize (L_Arbre : in out Arbre); 
  procedure Liberer is new 
             Ada.Unchecked_deallocation (Noeud'Class, Pt_Noeud);
 end Arbres_Binaires;
