--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--            PAQUETAGE DES FICHIERS BALANCES               --
--                                                          --
-- Note: un B-Arbre d'ordre m est un arbre general de       --
--    recherche, tel que:                                   --
--        toutes les feuilles sont au mme niveau           --
--        tout noeud sauf la racine contient au moins m     --
--            valeurs                                       --
--        tout noeud contient au plus 2m valeurs            --
--    Le paquetage implante un B-Arbre dans un fichier sur  --
--        disque, et fournit les operations de creation,    --
--        ouverture, fermeture destruction habituelles sur  --
--        un fichier, ainsi que les operations de           --
--        positionnement, de parcours, de consultation et   --
--        de modification des elements du fichiers          --
--        Les autres operations sont dans des unites enfant --
-- preconditions                                            --
--    Positionner (File, Sur): il doit exister dans le      --
--        fichier un element ayant pour cle Sur             --
--    Passer_Au_Suivant (File), Contenu_Courant(File):      --
--        non A_La_Fin (File)                               --
--    Changer_Courant (File, Val): non A_La_Fin (File)      --
--        et La_Cle(Contenu_Courant(File)) = La_Cle(Val)    --
-- proprietes fonctionnelles                                --
--    Creer (File, Mode, Name) cree un fichier balance      --
--        Contenu avec Val, et Sous_Arbres a vide           --
--    Ouvrir (File, Mode, Name) ouvre un fichier balance    --
--    Fermer (File) ferme le fichier                        --
--    Detruire (File) detruit le fichier                    --
--    Positionner (File, Sur) positionne le fichier sur     --
--        l'element de cle Sur                              --
--    Positionner_Sur_Premier (File) positionne le fichier  --
--        sur le premier element du fichier, i.e. celui de  --
--        plus petite cle                                   --
--    Passer_Au_Suivant (File) positionne le fichier sur    --
--        l'element suivant du fichier, dans l'ordre des    --
--        cles croissantes                                  --
--    A_La_Fin (File) est vrai si on est au bout du fichier --
--    Contenu_Courant (File) retourne l'element sur lequel  --
--        le fichier est positionne                         --
--    Changer_Courant (File, Val) change la valeur de       --
--        l'element sur lequel le fichier est positionne    --
--        longueur(Sous_Arbres) = 0 ou longueur(Contenu)+1  --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Arbres_Balances;
with Ada.Direct_IO;
with Ada.IO_Exceptions;
with Ada.Unchecked_Deallocation;
generic
  Ordre : in POSITIVE := 100;              -- ordre du B-Arbre
  type Element is private;
  type Cle is private;
  with function La_Cle (E : in Element) return Cle is <>;
  with function "<"(U, V : in Cle) return Boolean is <>;
package Balance_IO is
  type File_Type is limited private;
  type File_Mode is (In_File, Inout_File);
  procedure Creer (File : in out File_Type;
                    Mode : in File_Mode := Inout_File;
                    Name : in String := "");
  procedure Ouvrir (File : in out File_Type;
                     Mode : in File_Mode := Inout_File;
                     Name : in String := "");
  procedure Fermer (File : in out File_Type);
  procedure Detruire (File : in out File_Type);
  procedure Positionner (File : in out File_Type; Sur : in Cle);
  procedure Positionner_Sur_Premier (File : in out File_Type);
  procedure Passer_Au_Suivant (File : in out File_Type);
  function A_La_Fin (File : in File_Type) return Boolean;
  function Contenu_Courant (File : in File_Type) 
                                             return Element;
  procedure Changer_Courant (File : in File_Type;
                              Val : in Element);
  Cle_Existante : exception;
  Cle_Inconnue : exception;
  Cle_Changee : exception;
  Status_Error: exception renames Ada.IO_Exceptions.Status_Error;
  Mode_Error: exception renames Ada.IO_Exceptions.Mode_Error;
  Name_Error: exception renames Ada.IO_Exceptions.Name_Error;
  Use_Error: exception renames Ada.IO_Exceptions.Use_Error;
  Device_Error: exception renames Ada.IO_Exceptions.Device_Error;
  End_Error: exception renames Ada.IO_Exceptions.End_Error;
  Data_Error: exception renames Ada.IO_Exceptions.Data_Error;
private
  subtype Id_Noeud is Natural;
  Degre_Max : constant Positive := 2 * Ordre;
  package N_B is
    new Arbres_Balances (Degre_Max => Degre_Max + 1,
                            Id_Noeud => Id_Noeud,
                            Element => Element,
                            Cle => Cle); 
  package IO is new Ada.Direct_IO (Element_Type => N_B.Type_Noeud); 
  type Descr_Bloc;
  type Pt_Descr_Bloc is access Descr_Bloc;
  type File_Type is 
    record 
      File : IO.File_Type;
      Bloc_Courant : Pt_Descr_Bloc := null;
      Frere: Pt_Descr_Bloc := null;
    end record;
  type Descr_Bloc is
    record
      Contenu : N_B.Type_Noeud;             -- contenu d'un nud
      Indice : Positive;             -- position dans le nud
      Num_Bloc : Natural := 0;     -- position relative fichier
      Precedant, Suivant : Pt_Descr_Bloc := null; -- chemin
    end record;
  Trouve : Boolean;       -- cl trouve au cours de la recherche
  procedure Liberer is new 
       Ada.Unchecked_Deallocation (Descr_Bloc, Pt_Descr_Bloc);
  procedure Echanger_Avec_Frere (File: in out File_Type;
                                  Noeud : in out Pt_Descr_Bloc);
  procedure Lire_Fils(File:in out File_Type);
  procedure Lire_Frere (File : in out File_Type;
                         Noeud : in Pt_Descr_Bloc;
                         Depl : in Integer);
  procedure Ecrire (File : in File_Type; P : in Pt_Descr_Bloc);
  function Allouer_Bloc (File : in File_Type) return Positive;
  procedure Bord_Gauche (File: in out File_Type);
  procedure Valeur_Suivante (File: in out File_Type);
  procedure Recherche_Chemin (File : in out File_Type; 
                               C : in Cle);
end Balance_IO;
