--------------------------------------------------------------
--                                                          --
--            MODULE DE STRUCTURES DE DONNEES               --
--                                                          --
--           PAQUETAGE DES FICHIERS DE HACHAGE              --
--                                                          --
-- Note: le principe du hachage est de partitionner les     --
--    elements en petits groupes au moyen d'une fonction    --
--    qui sert a localiser ces groupes                      --
--    Le paquetage implante le hachage dans un fichier sur  --
--        disque, et fournit les operations de creation,    --
--        ouverture, fermeture destruction habituelles sur  --
--        un fichier, ainsi que les operations de           --
--        positionnement, de parcours, de consultation,     --
--        de modification, dadjonction et de suppression   --
--        des elements du fichiers                          --
-- preconditions                                            --
--    Positionner (File, Sur): il doit exister dans le      --
--        fichier un element ayant pour cle Sur             --
--    Passer_Au_Suivant (File), Contenu_Courant(File),      --
--        Supprimer_Courant (File): non A_La_Fin (File)     --
--    Changer_Courant (File, Val): non A_La_Fin (File)      --
--        et La_Cle(Contenu_Courant(File)) = La_Cle(Val)    --
--    Ajouter (File, Val): il ne doit pas exister dans le   --
--        fichier un element ayant pour cle La_Cle(Val)     --
-- proprietes fonctionnelles                                --
--    Creer (File, Mode, Name) cree un fichier balance      --
--        Contenu avec Val, et Sous_Arbres a vide           --
--    Ouvrir (File, Mode, Name) ouvre un fichier balance    --
--    Fermer (File) ferme le fichier                        --
--    Detruire (File) detruit le fichier                    --
--    Positionner (File, Sur) positionne le fichier sur     --
--        l'element de cle Sur                              --
--    Positionner_Sur_Premier (File) positionne le fichier  --
--        sur le premier element du fichier: le premier de  --
--        la plus petite valeur de hachage                  --
--    Passer_Au_Suivant (File) positionne le fichier sur    --
--        l'element suivant du fichier, dans l'ordre de     --
--        rangement des elements pour une mme valeur de    --
--        hachage, et dans lordre croissant de ces valeurs --
--    A_La_Fin (File) est vrai si on est au bout du fichier --
--    Contenu_Courant (File) retourne l'element sur lequel  --
--        le fichier est positionne                         --
--    Changer_Courant (File, Val) change la valeur de       --
--        l'element sur lequel le fichier est positionne    --
--        longueur(Sous_Arbres) = 0 ou longueur(Contenu)+1  --
--    Ajouter (File, Val) ajoute l'element Val au fichier,  --
--        le fichier est positionne ensuite sur cet element --
--    Supprimer_Courant(File) supprime l'element du fichier --
--        sur lequel on etait positionne, et positionne le  --
--        fichier sur l'element suivant (Passer_Au_Suivant) --
--                                                          --
-- Auteur: Christian CARREZ,                                --
-- Structures de donnees en Java, C++ et Ada 95, Dunod, 1997--
-- Institution: CNAM 292 rue Saint Martin, 75141 Paris 03   --
-- Derniere modification: 1 juillet 1997                    --
--------------------------------------------------------------

with Listes_Contigues;
with Ada.Direct_IO;
with Ada.IO_Exceptions;
generic
  Taille : in Positive := 20;	-- nombre d'lments par bloc
  type Espace is range<>; --espace de la fonction de hachage
  type Element is private;
  type Cle is private;
  with function La_Cle (E : in Element) return Cle is <>;
  with function Hachage (C : in Cle) return Espace is <>;
package Hachage_IO is
  type File_Type is limited private;
  type File_Mode is (In_File, Inout_File);
  procedure Creer (File : in out File_Type;
					Mode : in File_Mode := Inout_File;
					Name : in String := "");
  procedure Ouvrir (File : in out File_Type;
					 Mode : in File_Mode := Inout_File;
					 Name : in String := "");
  procedure Fermer (File : in out File_Type);
  procedure Detruire (File : in out File_Type);
  procedure Positionner (File : in out File_Type; Sur : in Cle);
  procedure Positionner_Sur_Premier (File : in out File_Type);
  procedure Passer_Au_Suivant (File : in out File_Type);
  function A_La_Fin (File : in File_Type) return Boolean;
  function Contenu_Courant (File : in File_Type) 
                                             return Element;
  procedure Changer_Courant (File : in out File_Type;
                              Val : in Element);
  procedure Ajouter (File : in out File_Type; Val : in Element);
  procedure Supprimer_Courant (File : in out File_Type);
  Cle_Existante : exception;
  Cle_Inconnue : exception;
  Cle_Changee : exception;
  Status_Error: exception renames Ada.IO_Exceptions.Status_Error;
  Mode_Error: exception renames Ada.IO_Exceptions.Mode_Error;
  Name_Error: exception renames Ada.IO_Exceptions.Name_Error;
  Use_Error: exception renames Ada.IO_Exceptions.Use_Error;
  Device_Error: exception renames Ada.IO_Exceptions.Device_Error;
  End_Error: exception renames Ada.IO_Exceptions.End_Error;
  Data_Error: exception renames Ada.IO_Exceptions.Data_Error;
private
  package L_E is new Listes_Contigues (Element => Element);
  type T_Bloc is
	record
	  Contenu : L_E.Liste (Taille);
	  Suivant : Natural := 0;
	end record;
  package IO is new Ada.Direct_IO (Element_Type => T_Bloc);
  use L_E, IO;
  subtype T_Num_B_Hach is Integer range 1..Espace'pos(Espace'Last)
                                    - Espace'pos (Espace'First) + 2;
  type File_Type is 
    record 
      File : IO.File_Type;
      Bloc: T_Bloc;
      Num_Bloc : Natural := 0;
      Indice : Positive;
      Hachage_Courant : T_Num_B_Hach := T_Num_B_Hach'Last;
    end record;
end Hachage_IO;
